@@ -1,5 +1,5 @@
 use anyhow::Result;
-use log::debug;
+use log::info;
 use rustc_hir::{
   intravisit::{self, NestedVisitorMap, Visitor},
   itemlikevisit::ItemLikeVisitor,
@@ -9,31 +9,36 @@ use rustc_middle::{hir::map::Map, ty::TyCtxt};
 use rustc_span::Span;
 use std::time::Instant;
 
-pub trait FlowistryOutput: Send + Sync {
-  fn empty() -> Self;
+use crate::core::utils::elapsed;
+
+use super::utils::block_timer;
+
+pub trait FlowistryOutput: Send + Sync + Default {
   fn merge(&mut self, other: Self);
 }
 
 pub trait FlowistryAnalysis: Send + Sync + Sized {
   type Output: FlowistryOutput;
 
-  fn locations(&self, tcx: TyCtxt) -> Vec<Span>;
+  fn locations(&self, tcx: TyCtxt) -> Result<Vec<Span>>;
   fn analyze_function(&mut self, tcx: TyCtxt, body_id: BodyId) -> Result<Self::Output>;
 
   fn run(self, compiler_args: &[String]) -> Result<Self::Output> {
     let mut compiler_args = compiler_args.to_vec();
 
     compiler_args.extend(
-      "-Z identify-regions -Z polonius -Z mir-opt-level=0 -A warnings"
-        .split(" ")
+      "-Z identify-regions -Z mir-opt-level=0 -A warnings"
+        .split(' ')
         .map(|s| s.to_owned()),
     );
 
     let mut callbacks = Callbacks {
       analysis: Some(self),
       output: None,
+      rustc_start: Instant::now(),
     };
 
+    info!("Starting rustc analysis...");
     rustc_driver::RunCompiler::new(&compiler_args, &mut callbacks)
       .run()
       .unwrap();
@@ -53,8 +58,8 @@ impl<A> VisitorContext<'_, A>
 where
   A: FlowistryAnalysis,
 {
-  fn analyze(&mut self, body_span: Span, body_id: BodyId) {
-    if !self.locations.iter().any(|span| body_span.contains(*span)) {
+  fn analyze(&mut self, item_span: Span, body_id: BodyId) {
+    if !self.locations.iter().any(|span| item_span.contains(*span)) {
       return;
     }
 
@@ -62,12 +67,11 @@ where
     let analysis = &mut self.analysis;
     take_mut::take(&mut self.output, move |output| {
       output.and_then(move |mut output| {
-        let start = Instant::now();
+        let fn_name = tcx.def_path_debug_str(tcx.hir().body_owner_def_id(body_id).to_def_id());
+        let timer_name = format!("Flowistry ({})", fn_name);
+        let _timer = block_timer(&timer_name);
+
         let new_output = analysis.analyze_function(tcx, body_id)?;
-        debug!(
-          "Finished in {} seconds",
-          start.elapsed().as_nanos() as f64 / 1e9
-        );
         output.merge(new_output);
         Ok(output)
       })
@@ -85,8 +89,13 @@ impl<A: FlowistryAnalysis> Visitor<'tcx> for AnalysisItemVisitor<'_, 'tcx, A> {
   }
 
   fn visit_nested_body(&mut self, id: BodyId) {
-    intravisit::walk_body(self, self.0.tcx.hir().body(id));
-    self.0.analyze(self.0.tcx.hir().span(id.hir_id), id);
+    let tcx = self.0.tcx;
+    intravisit::walk_body(self, tcx.hir().body(id));
+    let header_span = tcx.def_span(tcx.hir().body_owner_def_id(id));
+    let body_span = tcx.hir().span(id.hir_id);
+    let full_span = header_span.to(body_span);
+
+    self.0.analyze(full_span, id);
   }
 }
 
@@ -110,18 +119,23 @@ impl<A: FlowistryAnalysis> ItemLikeVisitor<'tcx> for AnalysisVisitor<'tcx, A> {
 struct Callbacks<A: FlowistryAnalysis> {
   analysis: Option<A>,
   output: Option<Result<A::Output>>,
+  rustc_start: Instant,
 }
 
 impl<A: FlowistryAnalysis> rustc_driver::Callbacks for Callbacks<A> {
-  fn after_analysis<'tcx>(
+  // TODO: does this need to be after_analysis? or can we do after_parsing
+  //   after limited testing this seems to work fine... and is WAY faster
+  fn after_parsing<'tcx>(
     &mut self,
     _compiler: &rustc_interface::interface::Compiler,
     queries: &'tcx rustc_interface::Queries<'tcx>,
   ) -> rustc_driver::Compilation {
+    elapsed("rustc", self.rustc_start);
+
     queries.global_ctxt().unwrap().take().enter(|tcx| {
       let analysis = self.analysis.take().unwrap();
-      let locations = analysis.locations(tcx);
-      let output = Ok(A::Output::empty());
+      let locations = analysis.locations(tcx).unwrap();
+      let output = Ok(A::Output::default());
       let mut visitor = AnalysisVisitor(VisitorContext {
         tcx,
         locations,
